#!/usr/bin/perl

# disk_stats.pl
#
# Written by Jon Smith <smithj@thelazysysadmin.net>
#
# With concepts and some code snippets taken from
# Mark Round <cacti@markround.com>
# http://www.markround.com/archives/48-Linux-iostat-monitoring-with-Cacti.html

use strict;

use Tie::IxHash;

use constant debug => 0;
my $base_oid = ".1.3.6.1.4.1.2021.51";
my %stats = ();
#my %data;

tie (my %data, "Tie::IxHash");

open (DF_PIPE, "df -l|");
my $i = 0;
while (<DF_PIPE>) {
        chomp;
        my @dfField = split(' ', $_);

        $dfField[0] =~ s/\/dev\///g;

        $data{$dfField[0]}{"dev"} = $dfField[0];
        $data{$dfField[0]}{"name"} = $dfField[5];
        $data{$dfField[0]}{"count"} = 1;

        $i++;
}
close(DF_PIPE);


open (VMSTAT_PIPE, "vmstat -d|");
while (<VMSTAT_PIPE>) {
        chomp;
        my @vmField = split(' ', $_);

        if (exists($data{$vmField[0]})) {
                $data{$vmField[0]}{"readms"} = $vmField[4];
                $data{$vmField[0]}{"writems"} = $vmField[8];
                $data{$vmField[0]}{"count"}++;
        }
}
close(VMSTAT_PIPE);

open (IOSTAT_PIPE, "iostat -dk|");
while (<IOSTAT_PIPE>) {
        chomp;
        my @ioField = split(' ', $_);

        if (exists($data{$ioField[0]})) {
                $data{$ioField[0]}{"kb_read"} = $ioField[4] * 1024;
                $data{$ioField[0]}{"kb_wrtn"} = $ioField[5] * 1024;
                $data{$ioField[0]}{"count"}++;
        }
}
close (IOSTAT_PIPE);

open (IOSTAT_PIPE, "iostat -dkx|");
while (<IOSTAT_PIPE>) {
        chomp;
        my @ioField = split(' ', $_);

        if (exists($data{$ioField[0]})) {
                $data{$ioField[0]}{"rs"} = $ioField[3];
                $data{$ioField[0]}{"ws"} = $ioField[4];
                $data{$ioField[0]}{"avgqu-sz"} = $ioField[8];
                $data{$ioField[0]}{"await"} = $ioField[9];
                $data{$ioField[0]}{"svctm"} = $ioField[10];
                $data{$ioField[0]}{"util"} = $ioField[11];
                $data{$ioField[0]}{"count"}++;
        }
}
close (IOSTAT_PIPE);

my $i = 1;
my $item;
foreach $item (keys %data) {
        if ($data{$item}{"count"} == 4) {
                $stats{"$base_oid.1.$i"} = $i;
                $stats{"$base_oid.2.$i"} = $data{$item}{"dev"};
                $stats{"$base_oid.3.$i"} = $data{$item}{"name"};
                $stats{"$base_oid.4.$i"} = $data{$item}{"readms"};
                $stats{"$base_oid.5.$i"} = $data{$item}{"writems"};
                $stats{"$base_oid.6.$i"} = $data{$item}{"rs"};
                $stats{"$base_oid.7.$i"} = $data{$item}{"ws"};
                $stats{"$base_oid.8.$i"} = $data{$item}{"kb_read"};
                $stats{"$base_oid.9.$i"} = $data{$item}{"kb_wrtn"};
                $stats{"$base_oid.10.$i"} = $data{$item}{"avgqu-sz"};
                $stats{"$base_oid.11.$i"} = $data{$item}{"await"};
                $stats{"$base_oid.12.$i"} = $data{$item}{"svctm"};
                $stats{"$base_oid.13.$i"} = $data{$item}{"util"};
                $i++;
        }
}

my $req;

my $mode = shift(@ARGV);
if ( $mode eq "-g" ) {
    $req = shift(@ARGV);
    getoid($req);
}
elsif ( $mode eq "-n" ) {
    $req = shift(@ARGV);
    my $next = getnextoid($req);
    getoid($next);
}
else {
    $req = $mode;
    getoid($req);
}

sub getoid {
    my $oid = shift(@_);
    print "Fetching oid : $oid\n" if (debug);
    if ( $oid =~ /^$base_oid\.(\d+)\.(\d+).*/ && exists( $stats{$oid} ) ) {
        print $oid. "\n";
        if ( $1 == 1 ) {
            print "integer\n";
        }
        else {
            print "string\n";
        }

        print $stats{$oid} . "\n";
    }
}

sub getnextoid {
    my $first_oid = shift(@_);
    my $next_oid  = '';
    my $count_id;
    my $index;

    if ( $first_oid =~ /$base_oid\.(\d+)\.(\d+).*/ ) {
        print("getnextoid($first_oid): index: $2, count_id: $1\n") if (debug);
        if ( $2 + 1 >= $i ) {
            $count_id = $1 + 1;
            $index    = 1;
        }
        else {
            $index    = $2 + 1;
            $count_id = $1;
        }
        print(
            "getnextoid($first_oid): NEW - index: $index, count_id: $count_id\n"
        ) if (debug);
        $next_oid = "$base_oid.$count_id.$index";
    }
    elsif ( $first_oid =~ /$base_oid\.(\d+).*/ ) {
        $next_oid = "$base_oid.$1.1";
    }
    elsif ( $first_oid eq $base_oid ) {
        $next_oid = "$base_oid.1.1";
    }
    print("getnextoid($first_oid): returning $next_oid\n") if (debug);
    return $next_oid;
}
